package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration
import com.smith9.geiger.client.beans.Stat
import groovy.xml.MarkupBuilder
import com.smith9.geiger.client.beans.Count
import org.joda.time.DateTime

/**
 * Created: 7/03/12
 * @author david
 */
class CountConverter implements LoaderConverter {
    String toXml(List<Stat> stats) {
        StringWriter writer = new StringWriter()
        MarkupBuilder builder = new MarkupBuilder(writer)
        builder.counts() {
            stats.each { durationNode ->
                count(toMap(durationNode))
            }
        }
        return writer.toString()
    }

    static Map<String, String> toMap(Count count) {
        assert count.periodStart, "periodStart is mandatory"
        Map<String, String> properties = [:]
        properties.put("category", count.category)
        properties.put("subCategory", count.subCategory)
        properties.put("count", count.count)
        properties.put("period", count.period)
        properties.put("periodStart", count.periodStart.toString("yyyy-MM-dd HH:mm:ss.SSS"))
        properties.put("productName", count.productName)
        if(count.environment != null) {
            properties.put("environment", count.environment)
        }
        if(count.subProduct != null) {
            properties.put("subProduct", count.subProduct)
        }
        if(count.failure != null) {
            properties.put("failure", count.failure)
        }
        if(count.min != null) {
            properties.put("min", count.min)
        }
        if(count.max != null) {
            properties.put("max", count.max)
        }
        if(count.sum != null) {
            properties.put("sum", count.sum)
        }
        return properties
    }

    String getLoadMethod() {
        return "counts"
    }
}
