package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration
import org.apache.commons.httpclient.methods.PostMethod
import org.apache.commons.httpclient.HttpClient

import org.joda.time.DateTime

import org.slf4j.Logger
import org.slf4j.LoggerFactory
import com.smith9.geiger.client.beans.Stat
import com.smith9.geiger.client.beans.Count
import com.smith9.geiger.client.beans.CountList
import simplygroovy.bean.Period

/**
 * Created: Mar 17, 2011
 * @author david
 */

class GeigerClient implements LoaderClient {
    private static final Logger LOG = LoggerFactory.getLogger(GeigerClient.class)
    private URL url
    private long totalSent
    private long totalDuration
//    private long batchesSent
    private HttpClient client

    public static void main(String[] args) {
        boolean loadDurations = false
        if (loadDurations) {
            List<Duration> durations = [
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 2, measure: "end to end", grouping: "1234567", reference: "123:1", failure: false, metadata: [start: "1", end: "5"]),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 2, measure: "render time", grouping: "1234567", reference: "123:2", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 6, measure: "render time", grouping: "123", reference: "123:3", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 4, measure: "end to end", grouping: "1234567", reference: "123:4", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 7, measure: "end to end", grouping: "123", reference: "123:5", failure: true, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", batch: true, batchType: "Other", startTime: new DateTime(), duration: 6, measure: "end to end", grouping: "1234567", reference: "123:6", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", batch: true, batchType: "Other", startTime: new DateTime(), duration: 7, measure: "end to end", grouping: "1234567", reference: "123:7", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "ProductA", batch: true, batchType: "BatchA", startTime: new DateTime(), duration: 10, measure: "end to end", grouping: "1234567", reference: "123:8", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "ProductA", batch: true, batchType: "BatchA", startTime: new DateTime(), duration: 29, measure: "end to end", grouping: "1234567", reference: "123:9", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5']),
                new Duration(environment: "SOTL", product: "EasyDoc", batch: true, batchType: "Other", startTime: new DateTime(), duration: 8, measure: "end to end", grouping: "1234567", reference: "123:10", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5'])
            ]

            new GeigerClient(new URL("http://localhost:8080/geiger")).send(durations)
        } else {
            GeigerClient client = new GeigerClient(new URL("http://localhost:8080/geiger"))
            BatchingGeigerClient batchingClient = new BatchingGeigerClient(100, client)
            LinkedHashMap<String, Integer> limitsPerHour = [0: 1, 1: 1, 2: 1, 3: 2, 4: 2, 5: 3, 6: 3, 7: 14, 8: 22, 9: 44, 10: 88, 11: 92, 12: 66, 13: 55, 14: 82, 15: 99, 16: 62, 17: 34, 18: 22, 19: 10, 20: 4, 21: 3, 22: 2, 23: 1]
            CountList counts = new CountList()
            DateTime start = new DateTime()
            int successCount = 0
            (2006..2008).each { int year ->
                println "year $year"
                (1..12).each { int month ->
                    println "month $month"
                    DateTime firstDayOfMonth = new DateTime(year, month, 1, 0, 0, 0, 0)
                    int lastDayOfMonth = firstDayOfMonth.dayOfMonth().withMaximumValue().dayOfMonth
                    (1..lastDayOfMonth).step(2) { int day ->
                        println "day $day"
                        (0..23).step(5) { int hour ->
                            int limitForHour = limitsPerHour[hour]
                            (0..59).step(15) { int minute ->
                                DateTime date = new DateTime(year, month, day, hour, minute, 0, 0)
                                (1..3).each { int product ->
                                    String subProductName = "Product$product"
                                    ['fax', 'email'].each { String targetType ->
                                        int count = (Math.random() * limitForHour) + 1
                                        boolean failure = ++successCount % 500 == 0
                                        if (failure) {
                                            int failCount = Math.random() * 5
                                            counts.add(new Count(category: "target type", subCategory: targetType, period: Period.MINUTE, periodStart: date,
                                                productName: "EasyDoc", subProduct: subProductName, count: failCount, environment: "PROD", min: failCount, max: failCount, sum: failCount, failure: true))

                                            if(counts.size() >= 100) {
                                                batchingClient.batch(counts.counts)
                                                batchingClient.batchRemaining()
                                                counts.clear()
                                            }
                                        }
                                        counts.add(new Count(category: "target type", subCategory: targetType, period: Period.MINUTE, periodStart: date,
                                            productName: "EasyDoc", subProduct: subProductName, count: count, environment: "PROD", min: count, max: count, sum: count, failure: false))

                                        if(counts.size() >= 100) {
                                            batchingClient.batch(counts.counts)
                                            batchingClient.batchRemaining()
                                            counts.clear()
                                        }
                                    }
                                }
                            }

                        }
                    }
                }
            }
        }
    }

    public GeigerClient(URL url) {
        this.url = url
        client = new HttpClient()
//        batchesSent = 0
    }

    Stat send(List<Stat> stats) {
        long start = System.currentTimeMillis()
        LoaderConverter converter = createConverter(stats)
        uploadStats(converter, stats)
        logTimeTaken(System.currentTimeMillis() - start, stats.size())
        return stats.last()
    }

    void logTimeTaken(Long duration, int count) {
        if (LOG.debugEnabled) {
            totalSent += count
            totalDuration += duration
            LOG.debug("Sent $count, 1 took ${duration / count / 1000}s, $totalSent averaged ${totalDuration / totalSent / 1000}s")
        }
    }

    private void uploadStats(LoaderConverter converter, List<Stat> stats) {
        PostMethod post = new PostMethod(url.toString() + "/" + converter.loadMethod)
        try {
            String xml = converter.toXml(stats)
            post.addParameter("xml", xml)
            int status = client.executeMethod(post)
            if (status != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                LOG.error("Error uploading stats to $url xml to post was: $xml")
                LOG.error("status code " + post.getStatusCode())
                LOG.error(post.getResponseBodyAsString())
            } else {
                // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
                cleanupResponseBody(post)
            }
        } finally {
            // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
            post.releaseConnection()
        }
    }

//    void cleanupClient() {
    //        if(batchesSent % 100 == 0) {
    //            LOG.debug("creating a new HttpClient to free memory/resources")
    //            client = new HttpClient()
    //        }
    //    }

    private void cleanupResponseBody(PostMethod post) {
        InputStream response = post.getResponseBodyAsStream()
        BufferedReader bufferedResponse = new BufferedReader(new InputStreamReader(response));
        String line;
        while ((line = bufferedResponse.readLine()) != null) {
        }
        bufferedResponse.close();
    }

    private LoaderConverter createConverter(List<Stat> stats) {
        if (stats.first() instanceof Duration) {
            return new DurationConverter()
        } else {
            return new CountConverter()
        }
    }
}